#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QGraphicsScene>
#include "freetypemodel.h"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    QPixmap* Pixmap;


    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

    void SetFreeTypeModel(TFreeTypeModel* freeTypeModel);

    int GetFaceWidth();
    int GetFaceHeight();

    int GetAreaWidth();
    int GetAreaHeight();

    int GetIntervalX();
    int GetIntervalY();

    QString GetSymbolList();

    void RefreshSheetHeightWidth();

    void SaveOutputEdit(std::vector<QString>& fontParamText, QString filename);

    void UnlockDrawButton();
    void UnlockSaveButton();

    void RefreshColorMarkers();

private:
    Ui::MainWindow *ui;
    TFreeTypeModel* FreeTypeModel;

public slots:
    void SetFontNameTextSlot(QString text);
    void SetLoadResultTextSlot(QString text);
    void InvalidatePixmap();
};

#endif // MAINWINDOW_H
